using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MeshVR;
using UnityEngine.UI;
using System.Security.AccessControl;

namespace JayJayWon
{
    public static class PC
    {
        public static bool patreonContentEnabled = true;

        public static void AppPresetSuppressScaleLoad(Atom atom, string presetFilePath, bool mergeLoad)
        {

            string presetPackageName = "";
            
            if (presetFilePath.Contains(":")) presetPackageName = presetFilePath.Substring(0, presetFilePath.IndexOf(':'));

            JSONClass presetJSON = (JSONClass)SuperController.singleton.LoadJSON(SuperController.singleton.NormalizePath(presetFilePath));
            JSONArray presetStorablesJSON = (JSONArray)presetJSON["storables"];

            JSONStorableFloat atomScale = atom.GetStorableByID("rescaleObject").GetFloatJSONParam("scale");
            foreach (JSONClass storableJSON in presetStorablesJSON)
            {
                if (storableJSON["id"].Value == "rescaleObject") storableJSON["scale"].AsFloat = atomScale.val;

            }
            if (presetPackageName != "")
            {
                string presetJSONString = presetJSON.ToString();
                if (presetJSONString.Contains("SELF:"))
                {
                    presetJSONString = presetJSONString.Replace("SELF:", presetPackageName + ":");
                    presetJSON = (JSONClass)JSON.Parse(presetJSONString);
                }
            }

            JSONStorable presetJS = atom.GetStorableByID("AppearancePresets");
            PresetManager pm = presetJS.GetComponentInChildren<PresetManager>();
            atom.SetLastRestoredData(presetJSON, true, true);
            pm.LoadPresetFromJSON(presetJSON, mergeLoad);
        }

        public static void LoadRandomFromSelections(BrowserUI parentBrowser, int resourceType, string atomPresetResourceType, ResourcePrimaryActionUI primaryActionUI)
        {
            var nextRandomRVGE = parentBrowser.resourceSelectionsUI.GetResourceTypeSelections(resourceType, atomPresetResourceType).GetNextRandomRVGE();
            if (nextRandomRVGE != null) Globals.mvrScript.StartCoroutine(nextRandomRVGE.Activate(primaryActionUI));
        }
        public static void LoadRandomFromFilterResults(BrowserUI parentBrowser,  ResourcePrimaryActionUI primaryActionUI)
        {
            var nextRandomRVGE = parentBrowser.GetNextRandomRVGEfromFilterResults();
            if (nextRandomRVGE != null) Globals.mvrScript.StartCoroutine(nextRandomRVGE.Activate(primaryActionUI));
        }

        public static void LoadPluginAsPreset(Atom atom, string pluginFilePath, bool openPluginUI = false)
        {
            MVRPluginManager manager;

            if (atom.type == "SessionPluginManager") manager = Globals.mvrScript.manager;
            else manager = atom.GetStorableByID("PluginManager") as MVRPluginManager;

            JSONClass currentPlugins = manager.GetJSON(true, true, true);

            JSONClass pluginPresetJSON = new JSONClass();
            pluginPresetJSON["setUnlistedParamsToDefault"].AsBool = true;

            JSONArray pluginPresetStorablesJSON = new JSONArray();
            pluginPresetJSON["storables"] = pluginPresetStorablesJSON;

            JSONClass pluginMgrStorableJSON = new JSONClass();
            pluginPresetStorablesJSON.Add(pluginMgrStorableJSON);
            pluginMgrStorableJSON["id"] = "PluginManager";

            JSONClass pluginMgrPluginsListJSON = new JSONClass();
            pluginMgrStorableJSON["plugins"] = pluginMgrPluginsListJSON;

            List<string> pluginRefsOfFirstLoadedPlugin = new List<string>();

            string normalFilePath = SuperController.singleton.NormalizePath(pluginFilePath);
            pluginRefsOfFirstLoadedPlugin = PluginUtils.GetPluginRefsOfType(manager, normalFilePath);
            pluginMgrPluginsListJSON["plugin#0"] = normalFilePath;

            MergePluginPreset(atom, pluginPresetJSON);

            if (openPluginUI)
            {
                var finalPluginRefsOfFirstLoadedPlugin = PluginUtils.GetPluginRefsOfType(manager, normalFilePath);

                int pluginSlot = int.Parse(finalPluginRefsOfFirstLoadedPlugin.Except(pluginRefsOfFirstLoadedPlugin).First().Substring(7));

                SelectAndOpenPluginUI(atom, pluginSlot);
            }
        }

        private static void SelectAndOpenPluginUI(Atom atom, int pluginSlot)
        {
            if (atom.name == "CoreControl" || atom.type == "SessionPluginManager")
            {
                SuperController.singleton.ShowMainHUDMonitor();
                SuperController.singleton.activeUI = SuperController.ActiveUI.MainMenu;
            }
            else
            {
                SuperController.singleton.SelectController(atom.mainController, false, false, true);
                SuperController.singleton.ShowMainHUDMonitor();
            }

            SuperController.singleton.StartCoroutine(WaitForPluginUI(atom, pluginSlot));
        }
        private static IEnumerator WaitForPluginUI(Atom atom, int pluginSlot)
        {
            var expiration = Time.unscaledTime + 1f;
            UITabSelector selector;
            while (Time.unscaledTime < expiration)
            {
                yield return 0;
                if (atom.name == "CoreControl" || atom.type == "SessionPluginManager") selector = SuperController.singleton.mainMenuTabSelector;
                else selector = atom.gameObject.GetComponentInChildren<UITabSelector>();
                if (selector == null) continue;

                if (atom.type == "SessionPluginManager") SuperController.singleton.mainMenuTabSelector.SetActiveTab("TabSessionPlugins");
                else if (atom.name == "CoreControl") SuperController.singleton.mainMenuTabSelector.SetActiveTab("TabScenePlugins");
                else selector.SetActiveTab("Plugins");

                IEnumerator enumerator1 = selector.transform.GetEnumerator();
                while (enumerator1.MoveNext())
                {
                    UITab component = ((Component)enumerator1.Current).GetComponent<UITab>();

                    if ((UnityEngine.Object)component != (UnityEngine.Object)null)
                    {
                        foreach (var scriptUI in component.GetComponentsInChildren<MVRScriptUI>())
                        {
                            scriptUI.closeButton?.onClick.Invoke();
                        }

                        foreach (var scriptController in component.GetComponentsInChildren<MVRScriptControllerUI>())
                        {
                            if (scriptController.label.text.StartsWith("plugin#" + pluginSlot.ToString() + "_"))
                            {
                                scriptController.openUIButton?.onClick?.Invoke();
                                break;
                            }
                        }

                    }
                }
                yield break;
            }
        }

        private static void MergePluginPreset(Atom atom, JSONClass pluginPresetJSON)
        {
            PresetLockStore tempPresetLockStore = new PresetLockStore();
            if (atom.type == "Person") tempPresetLockStore.StorePresetLocks(atom);

            PresetManager pm;
            if (atom.name == "CoreControl")
            {
                if (atom.type == "SessionPluginManager") pm = atom.GetComponentInChildren<PresetManager>();
                else
                {
                    JSONStorable js = atom.GetStorableByID("PluginManagerPresets");
                    pm = js.GetComponentInChildren<PresetManager>();
                }
            }
            else
            {
                if (atom.type != "Person") pm = atom.GetComponentInChildren<PresetManager>();
                else
                {
                    JSONStorable js = atom.GetStorableByID("PluginPresets");
                    pm = js.GetComponentInChildren<PresetManager>();
                }
            }
            atom.SetLastRestoredData(pluginPresetJSON, true, true);
            pm.LoadPresetFromJSON(pluginPresetJSON, true);

            if (atom.type == "Person") tempPresetLockStore.RestorePresetLocks(atom);
        }

        public static JSONStorable GetPresetStorable(Atom atom, int resourceType)
        {
            if (resourceType == ResourceType.presetAppearance) return atom.GetStorableByID("AppearancePresets");
            if (resourceType == ResourceType.presetAnimation) return atom.GetStorableByID("AnimationPresets");
            if (resourceType == ResourceType.presetBreastPhys) return atom.GetStorableByID("FemaleBreastPhysicsPresets");
            if (resourceType == ResourceType.presetClothing) return atom.GetStorableByID("ClothingPresets");
            if (resourceType == ResourceType.presetGeneral || resourceType==ResourceType.presetAtom) return atom.GetStorableByID("Preset");
            if (resourceType == ResourceType.presetGlutePhysics) return atom.GetStorableByID("FemaleGlutePhysicsPresets");
            if (resourceType == ResourceType.presetHair) return atom.GetStorableByID("HairPresets");
            if (resourceType == ResourceType.presetMorphs) return atom.GetStorableByID("MorphPresets");
            if (resourceType == ResourceType.presetPlugins) return atom.GetStorableByID("PluginPresets");
            if (resourceType == ResourceType.presetPose) return atom.GetStorableByID("PosePresets");
            if (resourceType == ResourceType.presetSkin) return atom.GetStorableByID("SkinPresets");
            return null;
        }
    }


}
